<?php
/**
*
* @package Sidebar
* @copyright (c) 01.01.2020 kaptangula
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace kaptangula\sidebar\event;

/**
* @ignore
*/
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
* Event listener
*/
class listener implements EventSubscriberInterface
{
	static public function getSubscribedEvents()
	{
		return array(
			'core.common'							=> 'common_setup',
			'core.user_setup'						=> 'load_language_on_setup',
			'core.index_modify_page_title'			=> 'topusers',
			'core.page_header'					=> 'newmessages',
		);
	}

	/* @var \phpbb\config\db_text $config_text */	/** @var \phpbb\auth\auth */
	protected $auth;
	
	/** @var \phpbb\config\config */
	protected $config;
	
	/** @var \phpbb\cache\driver\driver_interface */
	protected $cache;

	protected $config_text;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;

	/* @var \phpbb\controller\helper */
	protected $helper;

	/* @var \phpbb\template\template */
	protected $template;

	/** @var \phpbb\user */
	protected $user;

	/** @var string */
	protected $phpbb_root_path;
	
	/** @var string PHP file extension */
	protected $php_ext;

	/**
	* Constructor
	*
	* @param \phpbb\controller\helper	$helper		Controller helper object
	* @param \phpbb\template			$template	Template object
	*/
	public function __construct(
		\phpbb\auth\auth $auth,
		\phpbb\config\config $config,
		\phpbb\cache\service $cache,
		\phpbb\config\db_text $config_text,
		\phpbb\db\driver\driver_interface $db,
		\phpbb\controller\helper $helper,
		\phpbb\template\template $template,
		\phpbb\user $user,
		$phpbb_root_path,
		$php_ext
	)
	{
		$this->auth = $auth;
		$this->config = $config;
		$this->cache = $cache;
		$this->config_text = $config_text;
		$this->db = $db;
		$this->helper = $helper;
		$this->template = $template;
		$this->user = $user;
		$this->phpbb_root_path = $phpbb_root_path;
		$this->php_ext = $php_ext;
	}

	public function common_setup($event)
	{
		$sidebar_textb_data = $this->config_text->get_array(array(
			'sidebar_textb',		
		));

		$sidebar_text_data = $this->config_text->get_array(array(
			'sidebar_text',		
		));

		$this->template->assign_vars(array(
			'ENABLE_SIDEBLOCK'	=> $this->config['enable_sideblock'],
			'SIDEBAR_TEXTA'	=> $this->config['sidebar_texta'],
			'VIEWTOPIC_SIDEBLOCK'	=> $this->config['viewtopic_sideblock'],
			'S_AD_TYPE'				=> $this->config['ad_type'],
			'S_WIDTH_RATIO'			=> $this->config['width_ratio'],
			'S_STATISTICS_SIDE'		=> $this->config['statistics_side'],
			'S_ADSENSE_SIDE'		=> $this->config['adsense_side'],
			'S_SEARCH_SIDE'			=> $this->config['search_side'],
			'S_BIRTHDAYS_SIDE'		=> $this->config['birthdays_side'],
			'DISPLAY_TOP_USERS'		=> $this->config['display_top_users'],
			'S_LINK_SIDE'			=> $this->config['link_side'],																																									
			'LINK_SIDE_URL'			=> (isset($this->config['link_side_url'])) ? $this->config['link_side_url'] : '',		
			'LINK_SIDE_URLb'		=> '&lt;a&nbsp;href=&quot;'  . (($this->config['link_side_urlb']) ? $this->config['link_side_urlb'] : $this->config['link_side_urlb'] ) . '&lt;/a&gt;',
			'SIDEBAR_WIDTH'			=> (isset($this->config['sidebar_width'])) ? $this->config['sidebar_width'] : '',
			'SIDEBAR_TEXTB'			=> (isset($this->config['sidebar_textb'])) ? $this->config['sidebar_textb'] : '',
			'SIDEBAR_TEXT'			=> (isset($this->config['sidebar_text'])) ? $this->config['sidebar_text'] : '',
		));
	}

	public function load_language_on_setup($event)
	{
		$lang_set_ext = $event['lang_set_ext'];
		$lang_set_ext[] = array(
			'ext_name' => 'kaptangula/sidebar',
			'lang_set' => 'sidebar',
			'lang_set' => 'common',
		);
		$event['lang_set_ext'] = $lang_set_ext;
	}

	public function topusers($event)
	{
		/* Check cached data */
		if (($top_posts = $this->cache->get('_top_posts')) === false)
		{
			$topusers_nmb = $this->config['topusers_display_nmb'];

			$sql = 'SELECT u.username, u.user_id, u.user_colour, u.user_type, u.user_avatar, u.user_avatar_type, u.user_avatar_width, u.user_avatar_height, u.user_rank, u.user_posts, p.poster_id, p.post_time, COUNT(p.post_id) AS total_posts
				FROM ' . USERS_TABLE . ' u, ' . POSTS_TABLE . ' p
				WHERE u.user_id > ' . ANONYMOUS . '
					AND u.user_id = p.poster_id
						AND (u.user_type <> ' . USER_FOUNDER . ')
				GROUP BY u.user_id
				ORDER BY total_posts DESC, p.post_time DESC, p.post_id DESC';
			$result = $this->db->sql_query_limit($sql, $topusers_nmb);
			
			$top_posts = array();
	
			while ($row = $this->db->sql_fetchrow($result))
			{
				$top_posts[] = $row;
			}
	  
			$this->db->sql_freeresult($result);

			// Cache for 5 min.
			$this->cache->put('_top_posts', $top_posts, 300);
		}

		foreach( $top_posts as $row )
		{
			get_user_rank($row['user_rank'], $row['user_posts'], $row['rank_title'], $row['rank_image'], $row['rank_image_src']);

			/* Let's show the TPOTM then.. */
			$top_posts = (int) $row['total_posts'];

			/* only auth'd users can view the profile */
			$top_users_string = ($this->auth->acl_get('u_viewprofile')) ? get_username_string('full', $row['user_id'], $row['username'], $row['user_colour']) : get_username_string('no_profile', $row['user_id'], $row['username'], $row['user_colour']);

			/* Fresh install or when a new Month starts gives zero posts */
			$top_nobody = $this->user->lang['TOP_NOBODY'];

			$topuser_name = ($top_posts < 1) ? $top_nobody : $top_users_string;
			$avatar_size_size = $this->config['avatar_re_size'];

			$this->template->assign_block_vars('top_users',array(
				'TOPUSER_NAME'		=> $topuser_name,
				'RANK_TITLE'		=> $row['rank_title'],
				'TOP_POST'			=> $top_posts,
				'S_SEARCH_ACTION'	=> append_sid("{$this->phpbb_root_path}search.$this->php_ext", 'author_id=' . $row['user_id'] . '&amp;sr=posts'),
				'USER_AVATAR_THUMB'   	=>   ($row['user_avatar']) ? get_user_avatar($row['user_avatar'], $row['user_avatar_type'], ($row['user_avatar_width'] > $row['user_avatar_height']) ? $avatar_size_size : ($avatar_size_size / $row['user_avatar_height']) * $row['user_avatar_width'], ($row['user_avatar_height'] > $row['user_avatar_width']) ? $avatar_size_size : ($avatar_size_size / $row['user_avatar_width']) * $row['user_avatar_height']) : '',
			));
		}
		
		// USER MENU BLOCK
		if($this->user->data['is_registered'])
		{
			// Get user info from the browser sessions...
			$user_id = $this->user->data['user_id'];
			$username = $this->user->data['username'];

			// grab user data from database based on info retrieved above
			$sql = 'SELECT *
				FROM ' . USERS_TABLE . '
				WHERE ' . (($username) ? "username_clean = '" . $this->db->sql_escape(utf8_clean_string($username)) . "'" : "user_id = $user_id");
			$result = $this->db->sql_query($sql);
			$member = $this->db->sql_fetchrow($result);

			// set variables retrieved from database query
			$username = $member['username'];
			$user_id = (int) $member['user_id'];
			$colour = $member['user_colour'];

			$sql = 'SELECT rank_title, rank_image FROM ' . RANKS_TABLE . ' WHERE rank_id = ' . (int)$member['user_rank'] . ' LIMIT 1';
			$result = $this->db->sql_query($sql);
			if(($row = $this->db->sql_fetchrow($result)))
			{
				$rank_title = $row['rank_title'];
				$rank_img = $row['rank_image'];
			}
			else
			{
				$rank_title = $rank_img = '';
			}
			$rank = $this->db->sql_freeresult($result);

			// Assign specific vars
			$this->template->assign_vars(array(
				'USERNAME_FULL'      	=> get_username_string('full', $user_id, $username, $colour),
				'USERNAME'            	=> get_username_string('username', $user_id, $username, $colour),
				'USER_COLOR'         	=> $member['user_colour'],
				'USER_AVATAR_SIDE'  	=> get_user_avatar($member['user_avatar'], $member['user_avatar_type'], $member['user_avatar_width'], $member['user_avatar_height']),
				'RANK_TITLE_SIDE'   	=> $rank_title,
				'RANK_IMG_SRC' 			=> $this->phpbb_root_path . 'images/ranks/' . $rank_img,
				'U_VIEW_PROFILE'     	=> $this->phpbb_root_path.'memberlist.php?mode=viewprofile&u='.$user_id,
				'U_PRIVATE_MESSAGES' 	=> append_sid($this->phpbb_root_path.'ucp.' . $this->php_ext . '?i=pm&folder=inbox'),
				'U_LOGIN_LOGOUT'      	=> $this->phpbb_root_path.'ucp.'.$this->php_ext.'?mode=logout&subdir=1&sid='.$this->user->data['session_id'],
				'U_NEW_POSTS'         	=> append_sid($this->phpbb_root_path.'search.' . $this->php_ext . '?search_id=newposts'),
				'U_SELF_POSTS'       	=> append_sid($this->phpbb_root_path.'search.' . $this->php_ext . '?search_id=egosearch'),
			));
		}
		// END USER MENU BLOCK	
	}
	
	public function newmessages($event)
	{
		if( ( $newmessages = $this->cache->get('_recent_new_msg') ) === false )
		{	
			$msg_read_ary = array();
			$forum_read_ary = $this->auth->acl_getf('f_read');

			foreach ($forum_read_ary as $forum_id => $allowed)
			{
				if ($allowed['f_read'])
				{
					$msg_read_ary[] = (int) $forum_id;
				}
			}
			$msg_read_ary = array_unique($msg_read_ary);

			$msgnew = array(
				'SELECT'	=> 'p.post_id, p.post_visibility, p.poster_id, p.post_time, p.post_subject, p.forum_id, p.post_text, p.topic_id, p.bbcode_bitfield, p.bbcode_uid, p.enable_bbcode, p.enable_smilies, p.enable_magic_url, u.user_id, u.username, u.user_colour, u.user_type, u.user_avatar, u.user_avatar_type, u.user_avatar_width, u.user_avatar_height, t.topic_id, t.topic_title, t.forum_id, t.topic_last_post_id, t.topic_last_post_time',
				'FROM'		=> array(POSTS_TABLE => 'p'),
				'LEFT_JOIN'	=> array(
					array(
						'FROM'	=> array(TOPICS_TABLE => 't'),
						'ON'	=> 'p.topic_id = t.topic_id',
					),
					array(
						'FROM'	=> array(USERS_TABLE => 'u'),
						'ON'	=> 't.topic_last_poster_id = u.user_id',
					),
				),
				'WHERE'		=> 	'
				(
					p.topic_id  = t.topic_id
				)
					AND ( ' . $this->db->sql_in_set('t.forum_id', $msg_read_ary) . ')
					AND p.poster_id = u.user_id
					AND t.topic_status <> ' . ITEM_MOVED . '
					AND t.topic_visibility = 1
					AND t.forum_id <> 0',	
				'GROUP_BY'	=> 'post_id',
				'ORDER_BY'	=> 'post_time DESC',
			);

			$result = $this->db->sql_query_limit($this->db->sql_build_query('SELECT', $msgnew), 5);

			$newmessages = array();
			while ($row = $this->db->sql_fetchrow($result))
			{
				$newmessages[] = $row;
			}
	  
			$this->db->sql_freeresult($result);
			// Cache for 5 min.
			$this->cache->put('_recent_new_msg', $newmessages, 300);
		}
 
		foreach($newmessages as $row)
		{
	
			/* only auth'd users can view the profile */
			$top_users_string = ($this->auth->acl_get('u_viewprofile')) ? get_username_string('full', $row['user_id'], $row['username'], $row['user_colour']) : get_username_string('no_profile', $row['user_id'], $row['username'], $row['user_colour']);

			/* Let's show the TPOTM then.. */
			$new_msgs = (int) $row['post_text'];

			$avatar_size_size = $this->config['avatar_re_size'];
			$post_id = $row['post_id'];
			$uid = $bitfield = $options = '';
			$allow_bbcode = $allow_smilies = $allow_urls = true;
			$flags = (($allow_bbcode) ? OPTION_FLAG_BBCODE : 0) + (($allow_smilies) ? OPTION_FLAG_SMILIES : 0) + (($allow_urls) ? OPTION_FLAG_LINKS : 0);
			$message = generate_text_for_display($row['post_text'], $row['bbcode_uid'], $row['bbcode_bitfield'], $flags);
			$msg_chars  = '60';
			if ($msg_chars != 0 and utf8_strlen($message) > $msg_chars)
			{
				$allow_bbcode = $allow_smilies = $allow_urls = false;
				strip_bbcode($message, $row['bbcode_uid']);
				$msg_full = utf8_substr($message, 0, $msg_chars) . '...';
				$msg_full = $msg_full . '...'; 
				$post_link = append_sid("{$this->phpbb_root_path}viewtopic.$this->php_ext", "p=" . $post_id . "#p" . $post_id);
				$read_full = $this->user->lang['READ_FULL'];
			}
			else
			{
				$msg_full = $message;
				$post_link 	= '';
				$read_full 	= '';
			}
			$avatar_size_size = 30;

			$this->template->assign_block_vars('newest_msg',array(
				'LASTPOSTER_NAME'	=> $top_users_string,
				'TOPIC_TITLE'		=> $row['topic_title'],
				'DATE'				=> $this->user->format_date($row['post_time']),
				'TOP_MSG'			=> $msg_full,
				'L_READ_FULL'		=> $read_full,
				'U_READ_FULL'		=> $post_link,
				'POSTER_AVATAR' 	=>   ($row['user_avatar']) ? get_user_avatar($row['user_avatar'], $row['user_avatar_type'], ($row['user_avatar_width'] > $row['user_avatar_height']) ? $avatar_size_size : ($avatar_size_size / $row['user_avatar_height']) * $row['user_avatar_width'], ($row['user_avatar_height'] > $row['user_avatar_width']) ? $avatar_size_size : ($avatar_size_size / $row['user_avatar_width']) * $row['user_avatar_height']) : '',
			));
		}
	}
}